import './App.css';
import * as action from './actions'
import * as reducer from './reducers'

import thunk from 'redux-thunk';
import { useEffect, useState } from 'react';
import { createStore, combineReducers, applyMiddleware } from 'redux';
import { Provider, connect } from 'react-redux';
import { Link, Route, Router, Switch, Redirect } from 'react-router-dom';
import createHistory from 'history/createBrowserHistory'

export const getGQL = url =>
  (query, variables = {}) =>
    fetch(url, {
      method: 'POST',
      headers: {
        "Content-Type": "application/json",
        ...(localStorage.authToken ? { "Authorization": "Bearer " + localStorage.authToken } : {})
      },
      body: JSON.stringify({ query, variables })
    })
      .then(res => res.json())
      .then(data => {
        if (data.errors && !data.data) throw new Error(JSON.stringify(data.errors))
        return data.data[Object.keys(data.data)[0]]
      })

const history = createHistory()
const backendURL = "http://player.asmer.fs.a-level.com.ua"
export const gql = getGQL(backendURL + '/graphql')

const store = createStore(
  combineReducers(
    {
      promise: reducer.promiseReducer,
      auth: reducer.authReducer,
      //local: localStoreReducer(promiseReducer, 'locale')
    }
  ), applyMiddleware(thunk)
)

store.subscribe(() => console.log(store.getState()))
if(store.getState().auth?.token) {
  //history.push('/player')
  store.dispatch(action.actionGetUserData())
} else {
  history.push('/login')
}

export function jwtDecode(token) {
  try {
    let decoded = token.split('.')
    decoded = decoded[1]
    decoded = atob(decoded)
    decoded = JSON.parse(decoded)
    return decoded

  } catch (e) {
    return;
  }
}

const LoginForm = ({ loged, onLogin }) => {
  let [login, setLogin] = useState()
  let [password, setPassword] = useState()
  let [log, setLog] = useState()

  useEffect(() => {
    setLog(loged)
    if (log?.payload && localStorage.authToken) history.push('/player')
  }, [loged, log])

  return (
    <>
      <h1>Web-player</h1>
      <div>
        <h2>Log-in</h2>
        <input type="text" placeholder='Login' onChange={(e) => setLogin(e.target.value)} />
        <br />
        <input type="password" placeholder='Password' onChange={(e) => setPassword(e.target.value)} />
        <br />
        <small style={{ color: 'red' }}>{loged.status === 'REJECTED' || (loged.status === 'RESOLVED' && !loged.payload) ? 'invalid login or password' : ''}</small>
        <br />
        <button
          disabled={!password || !login}
          onClick={() => { onLogin(login, password) }}
        >Login</button>
        <p>- OR -</p>
        <Link to="/registration">Register new user</Link>
      </div>
    </>
  )
}

const LoginFormConnect = connect(state => ({ loged: state.promise.login || {} }), { onLogin: action.actionFullLogin })(LoginForm)

const RegisterForm = ({ onRegister }) => {
  let [login, setLogin] = useState()
  let [password, setPassword] = useState()
  let [password2, setPassword2] = useState()

  return (
    <>
      <h1>Web-player</h1>
      <div>
        <h2>Registration</h2>
        <input type="text" placeholder='Login' onChange={(e) => setLogin(e.target.value)} />
        <br />
        <input type="password" placeholder='Password' onChange={(e) => setPassword(e.target.value)} />
        <br />
        <input disabled={!password} type="password" placeholder='Repeat Password' onChange={(e) => setPassword2(e.target.value)} />
        <br />
        <small style={{ color: 'red' }}>{password2 && password2 !== password ? 'Passwords do not match' : ''}</small>
        <br />
        <button disabled={!password || !login || password2 !== password} onClick={() => onRegister(login, password)}>Register</button>
        <br />
        <Link to="/login">Back to log-in page</Link>
      </div>
    </>
  )
}
const RegisterFormConnect = connect(null, { onRegister: action.actionRegister })(RegisterForm)

const Player = ({ user, getUserData, onLogout }) => {
  let [userInfo, setUserInfo] = useState(user.payload)
  useEffect(()=> {
    setUserInfo(user.payload)
  },[user, userInfo])
  
  return (
    <>
      <header>Player</header>
      <div style={{ display: 'flex' }}>
        <aside style={{ border: '1px solid black', width: '20%' }}>
          <div
            style={{ border: '1px solid black', backgroundColor: 'red', color: 'white' }}
            onClick={() => { onLogout(); history.push('/login') }}
          >
            log-out[X]
          </div>

          <div style={{ display: 'flex', alignItems: 'center' }}>
            {/* avatar */}
            <img 
              width={100} 
              height={100}  
              style={{ border: '1px solid black' }}
              src={ userInfo?.avatar?.url ? backendURL + '/' + userInfo?.avatar?.url : ''}
              alt='avatar' 
            />
    
            <div>
              <span>{userInfo?.login || 'user'}</span>
            </div>
          </div>
          <div style={{ border: '1px solid black' }}>
            <h4>User playlists:</h4>
            <div style={{ border: '1px solid black' }}>Add plylist [+]</div>
            <ul>
              тут карочи линки будут
              <li>SOME ITEM</li>
              <li>SOME ITEM</li>
              <li>SOME ITEM</li>
              <li>SOME ITEM</li>
            </ul>
          </div>
        </aside>
        <main style={{ border: '1px solid black', width: '80%' }}>
          MAIN
          рут патх='путь' компонет=`компогнент отривоски`
        </main>
      </div>
      <footer> back stop forw</footer>
    </>
  )
}

const PlayerConnect = connect(
  state => ({
    user: state.promise.userData || {}
    //playlists: state.promise.getPlaylists || {}
  }),
  { 
    onLogout: action.actionAuthLogout,
    getUserData: action.actionGetUserData,
    //getPlaylists: actionGetPlaylists
  }
)(Player)


function App() {
  return (
    <Router history={history}>
      <Provider store={store}>
        <div className="App">
          <Switch>
            <Route path="/login" component={LoginFormConnect} exact />
            <Route path="/registration" component={RegisterFormConnect} exact />
            <Route path="/player" component={PlayerConnect} exact />
          </Switch>
        </div>
      </Provider>
    </Router>
  );
}

export default App;