import {jwtDecode, gql} from '../App'


const actionPending = name => ({ type: 'PROMISE', status: 'PENDING', name })
const actionResolved = (name, payload) => ({ type: 'PROMISE', status: 'RESOLVED', name, payload })
const actionRejected = (name, error) => ({ type: 'PROMISE', status: 'REJECTED', name, error })

export const actionPromise = (name, promise) =>
  async dispatch => {
    dispatch(actionPending(name))
    try {
      let payload = await promise
      dispatch(actionResolved(name, payload))
      return payload
    }
    catch (error) {
      dispatch(actionRejected(name, error))
    }
  }

export const actionAuthLogin = token => ({ type: 'AUTH_LOGIN', token })
export const actionAuthLogout = () => ({ type: 'AUTH_LOGOUT' })

export const actionLogin = (login, password) =>
  actionPromise('login', gql(`
  query log($login:String!, $password:String!) {
      login(login: $login, password: $password)
    }`, { login, password }))

export const actionFullLogin = (login = 'tst', password = '123') =>
  async dispatch => {
    let token = await dispatch(actionLogin(login, password))
    if (token) {
      await dispatch(actionAuthLogin(token))
      dispatch(actionGetUserData())
    }
  }

export const actionRegister = (login, password) =>
    actionPromise('registration', gql(`
        mutation register($login:String!, $password:String!) {
            createUser(login: $login, password: $password) {
                login, _id
            }
        }
        `, { login, password })
    )

export const actionFullRegister = (login = 'tst', password = '123') =>
  async dispatch => {
    await dispatch(actionRegister(login, password))
    await dispatch(actionFullLogin(login, password))
  }

export const actionGetUserData = () => {
    let _id = jwtDecode(localStorage.authToken).sub.id
    return (
        actionPromise('userData', gql(`
            query($userId: String!) {
                UserFindOne(query: $userId){
                    login, _id, avatar {_id, url, originalFileName}
                }
            }
        `, { userId: JSON.stringify([{_id}]) }))
    )
}

// const actionGetPlaylists = (userId = "5fe35e1ce926687ee86b0a3f") => {
//     actionPromise('getPlaylists', gql(`
//             query getPlaylistByOwnerId($ownerId:String!) {
//                 PlaylistFind(query: $ownerId) {
//                 _id, name
//                 }
//             }
//         `, { ownerId: JSON.stringify([{ ___owner: userId }]) })
//     )
// }

